/*
 * Copyright (C) 2020  Anthony Di Pietro
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file Configuration for
 * [npm-package-json-lint](https://npmpackagejsonlint.org/).
 *
 * @module @fablab/npm-package-lint-json-config
 * @author Anthony Di Pietro <anthony.dipietro@research.uwa.edu.au>
 * @copyright © 2020 Anthony Di Pietro
 * @license AGPL-3.0-or-later
 */

/**
 * Packages that are excepted from not being allowed to use git versions.
 *
 * @constant {Object}
 */
const gitVersionExceptions = [
	'jspsych',
];

/**
 * Packages that are excepted from having to use caret version specifications.
 *
 * @constant {Object}
 */
const caretVersionExceptions = [
	...gitVersionExceptions,
	'eslint',
	'@fablab/commitlint-config',
	'@fablab/eslint-config',
	'@fablab/npm-package-json-lint-config',
	'@fablab/stylelint-config',
];

/**
 * Packages that are excepted from having to use nonzero versions.
 *
 * @constant {Object}
 */
const versionZeroExceptions = [
	'eslint-plugin-no-use-extend-native',
	'eslint-plugin-xss',
	'html-loader',
	'http-server',
	'markdownlint-cli',
	'mersennetwister',
	'mini-css-extract-plugin',
	'ramda',
	'regenerator-runtime',
	'resource-hints-webpack-plugin',
	'sharp',
	'stylelint-at-rule-no-children',
	'webpack-shell-plugin',
];

module.exports = {
	'extends': 'npm-package-json-lint-config-default',
	'rules': {
		'require-author': 'error',
		'require-bin': 'off',
		'require-bugs': 'warning',
		'require-bundledDependencies': 'off',
		'require-config': 'off',
		'require-contributors': 'off',
		'require-cpu': 'off',
		'require-dependencies': 'warning',
		'require-description': 'error',
		'require-devDependencies': 'warning',
		'require-directories': 'off',
		'require-engines': 'warning',
		'require-files': 'off',
		'require-funding': 'warning',
		'require-homepage': 'warning',
		'require-keywords': 'error',
		'require-license': 'error',
		'require-main': 'error',
		'require-man': 'off',
		'require-module': 'off',
		'require-name': 'error',
		'require-optionalDependencies': 'off',
		'require-os': 'off',
		'require-peerDependencies': 'off',
		'require-preferGlobal': 'off',
		'require-private': 'error',
		'require-publishConfig': 'off',
		'require-repository-directory': 'off',
		'require-repository': 'error',
		'require-scripts': 'error',
		'require-types': 'off',
		'require-typings': 'off',
		'require-version': 'error',
		'bin-type': 'error',
		'bundledDependencies-type': 'error',
		'config-type': 'error',
		'cpu-type': 'error',
		'dependencies-type': 'error',
		'description-type': 'error',
		'devDependencies-type': 'error',
		'directories-type': 'error',
		'engines-type': 'error',
		'files-type': 'error',
		'homepage-type': 'error',
		'keywords-type': 'error',
		'license-type': 'error',
		'main-type': 'error',
		'man-type': 'error',
		'name-type': 'error',
		'optionalDependencies-type': 'error',
		'os-type': 'error',
		'peerDependencies-type': 'error',
		'preferGlobal-type': 'error',
		'private-type': 'error',
		'repository-type': 'error',
		'scripts-type': 'error',
		'version-type': 'error',
		'valid-values-author': [
			'warning',
			[
				'Anthony Di Pietro',
				'Simon Farrell',
			],
		],
		'valid-values-engines': [
			'warning',
			[
				{'node': '^10.0.0'},
				{'node': '^12.0.0'},
				{'node': '>=10'},
				{'node': '>=12'},
				{'node': '>=13'},
				{'node': '>=14'},
				{'node': '>=13.2.0'},
			],
		],
		'valid-values-license': [
			'warning',
			[
				'AFL-3.0',
				'AGPL-3.0-only',
				'AGPL-3.0-or-later',
				'Apache-2.0',
				'APSL-2.0',
				'Artistic-2.0',
				'BSD-3-Clause',
				'BSL-1.0',
				'CDDL-1.0',
				'CPAL-1.0',
				'CPL-1.0',
				'ECL-2.0',
				'EFL-2.0',
				'EPL-2.0',
				'EUDatagrid',
				'EUPL-1.2',
				'GPL-3.0-only',
				'GPL-3.0-or-later',
				'HPND',
				'Intel',
				'IPA',
				'IPL-1.0',
				'ISC',
				'LGPL-3.0-only',
				'LGPL-3.0-or-later',
				'LPL-1.02',
				'MIT',
				'MPL-2.0',
				'MS-PL',
				'MS-RL',
				'NCSA',
				'Nokia',
				'OFL-1.1',
				'OSL-3.0',
				'Python-2.0',
				'QPL-1.0',
				'RPSL-1.0',
				'SISSL',
				'Sleepycat',
				'SPL-1.0',
				'UPL-1.0',
				'W3C',
				'Zlib',
				'ZPL-2.0',
			],
		],
		'valid-values-name-scope': [
			'error',
			[
				'@adp-psych',
				'@fablab',
			],
		],
		'valid-values-private': [
			'warning',
			[
				false,
				true,
			],
		],
		'valid-values-publishConfig': 'off',
		'no-repeated-dependencies': 'error',
		'no-absolute-version-dependencies': 'warning',
		'no-archive-dependencies': 'error',
		'no-caret-version-dependencies': 'off',
		'no-file-dependencies': 'warning',
		'no-git-dependencies': [
			'warning',
			{'exceptions': gitVersionExceptions},
		],
		'no-restricted-dependencies': [
			'warning',
			['funding'],
		],
		'no-restricted-pre-release-dependencies': [
			'warning',
			['funding'],
		],
		'no-tilde-version-dependencies': 'off',
		'prefer-absolute-version-dependencies': 'off',
		'prefer-alphabetical-dependencies': 'warning',
		'prefer-caret-version-dependencies': [
			'warning',
			{'exceptions': caretVersionExceptions},
		],
		'prefer-no-version-zero-dependencies': [
			'warning',
			{'exceptions': versionZeroExceptions},
		],
		'prefer-tilde-version-dependencies': 'off',
		'no-absolute-version-devDependencies': 'warning',
		'no-archive-devDependencies': 'error',
		'no-caret-version-devDependencies': 'off',
		'no-file-devDependencies': 'warning',
		'no-git-devDependencies': [
			'warning',
			{'exceptions': gitVersionExceptions},
		],
		'no-restricted-devDependencies': [
			'warning',
			['funding'],
		],
		'no-restricted-pre-release-devDependencies': [
			'warning',
			['funding'],
		],
		'no-tilde-version-devDependencies': 'off',
		'prefer-absolute-version-devDependencies': 'off',
		'prefer-alphabetical-devDependencies': 'warning',
		'prefer-caret-version-devDependencies': [
			'warning',
			{'exceptions': caretVersionExceptions},
		],
		'prefer-no-version-zero-devDependencies': [
			'warning',
			{'exceptions': versionZeroExceptions},
		],
		'prefer-tilde-version-devDependencies': 'off',
		'prefer-alphabetical-bundledDependencies': 'warning',
		'prefer-alphabetical-optionalDependencies': 'warning',
		'prefer-alphabetical-peerDependencies': 'warning',
		'prefer-alphabetical-scripts': 'warning',
		'description-format': [
			'warning',
			{
				'requireCapitalFirstLetter': false,
				'requireEndingPeriod': false,
			},
		],
		'name-format': 'error',
		'version-format': 'error',
		'prefer-property-order': [
			'warning',
			[],
		],
		'no-duplicate-properties': 'error',
		'prefer-no-engineStrict': 'error',
		'prefer-no-dependencies': 'off',
		'prefer-no-devDependencies': 'off',
		'prefer-no-optionalDependencies': 'off',
		'prefer-no-peerDependencies': 'off',
	},
};
